/* Target-dependent code for GNU/Linux Super-H.

   Copyright (C) 2005, 2007, 2008, 2009 Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

#include "defs.h"
#include "osabi.h"

#include "gdb_assert.h"
#include "gdb_string.h"

#include "solib-svr4.h"
#include "symtab.h"
#include "regcache.h"
#include "regset.h"

#include "glibc-tdep.h"
#include "sh-tdep.h"

/* Copied from asm/ptrace.h, and made 64-bit safe:
   This struct defines the way the registers are stored on the
   kernel stack during a system call or other kernel entry.  */
struct sh_linux_pt_regs
  {
    uint32_t regs[16];
    uint32_t pc;
    uint32_t pr;
    uint32_t sr;
    uint32_t gbr;
    uint32_t mach;
    uint32_t macl;
    int32_t tra;
  };

/* Copied from sys/user.h, and made 64-bit safe.  */
struct sh_linux_user_fpu_struct
  {
    uint32_t fp_regs[16];
    uint32_t xfp_regs[16];
    uint32_t fpscr;
    uint32_t fpul;
  };

/* Supply register REGNUM from the buffer specified by GREGS and LEN
   in the general-purpose register set REGSET to register cache
   REGCACHE.  If REGNUM is -1, do this for all registers in REGSET.  */

static void
sh_linux_supply_gregset (const struct regset *regset,
		       struct regcache *regcache,
		       int regnum, const void *gregs, size_t len)
{
  struct gdbarch *gdbarch = get_regcache_arch (regcache);
  const struct sh_linux_pt_regs *regs = gregs;
  int i;

  gdb_assert (len >= sizeof (struct sh_linux_pt_regs));

  if (regnum == gdbarch_pc_regnum (gdbarch) || regnum == -1)
    regcache_raw_supply (regcache, gdbarch_pc_regnum (gdbarch),
			 &regs->pc);

  if (regnum == SR_REGNUM || regnum == -1)
    regcache_raw_supply (regcache, SR_REGNUM, &regs->sr);

  if (regnum == PR_REGNUM || regnum == -1)
    regcache_raw_supply (regcache, PR_REGNUM, &regs->pr);

  if (regnum == GBR_REGNUM || regnum == -1)
    regcache_raw_supply (regcache, PR_REGNUM, &regs->gbr);

  if (regnum == MACH_REGNUM || regnum == -1)
    regcache_raw_supply (regcache, MACH_REGNUM, &regs->mach);

  if (regnum == MACL_REGNUM || regnum == -1)
    regcache_raw_supply (regcache, MACL_REGNUM, &regs->macl);

  for (i = R0_REGNUM; i <= (R0_REGNUM + 15); i++)
    {
      if (regnum == i || regnum == -1)
	regcache_raw_supply (regcache, i, &regs->regs[i - R0_REGNUM]);
    }
}

/* Collect register REGNUM in the general-purpose register set
   REGSET. from register cache REGCACHE into the buffer specified by
   GREGS and LEN.  If REGNUM is -1, do this for all registers in
   REGSET.  */

static void
sh_linux_collect_gregset (const struct regset *regset,
			const struct regcache *regcache,
			int regnum, void *gregs, size_t len)
{
  struct gdbarch *gdbarch = get_regcache_arch (regcache);
  struct sh_linux_pt_regs *regs = gregs;
  int i;

  gdb_assert (len >= sizeof (struct sh_linux_pt_regs));

  if (regnum == gdbarch_pc_regnum (gdbarch) || regnum == -1)
    regcache_raw_collect (regcache, gdbarch_pc_regnum (gdbarch),
			  &regs->pc);

  if (regnum == SR_REGNUM || regnum == -1)
    regcache_raw_collect (regcache, SR_REGNUM, &regs->sr);

  if (regnum == PR_REGNUM || regnum == -1)
    regcache_raw_collect (regcache, PR_REGNUM, &regs->pr);

  if (regnum == GBR_REGNUM || regnum == -1)
    regcache_raw_collect (regcache, GBR_REGNUM, &regs->gbr);

  if (regnum == MACH_REGNUM || regnum == -1)
    regcache_raw_collect (regcache, MACH_REGNUM, &regs->mach);

  if (regnum == MACL_REGNUM || regnum == -1)
    regcache_raw_collect (regcache, MACL_REGNUM, &regs->macl);

  for (i = R0_REGNUM; i <= (R0_REGNUM + 15); i++)
    {
      if (regnum == i || regnum == -1)
	regcache_raw_collect (regcache, i, &regs->regs[i - R0_REGNUM]);
    }
}

/* Supply register REGNUM from the buffer specified by FPREGS and LEN
   in the floating point register set REGSET to register cache
   REGCACHE.  If REGNUM is -1, do this for all registers in REGSET.  */

static void
sh_linux_supply_fpregset (const struct regset *regset,
			  struct regcache *regcache,
			  int regnum, const void *fpregs, size_t len)
{
  struct gdbarch *gdbarch = get_regcache_arch (regcache);
  const struct sh_linux_user_fpu_struct *regs = fpregs;
  int fp0_regnum = gdbarch_fp0_regnum (gdbarch);
  int i;

  gdb_assert (len >= sizeof (struct sh_linux_user_fpu_struct));

  if (regnum == FPSCR_REGNUM || regnum == -1)
    regcache_raw_supply (regcache, FPSCR_REGNUM, &regs->fpscr);

  if (regnum == FPUL_REGNUM || regnum == -1)
    regcache_raw_supply (regcache, FPUL_REGNUM, &regs->fpul);

  for (i = fp0_regnum; i <= (fp0_regnum + 15); i++)
    {
      if (regnum == i || regnum == -1)
	regcache_raw_supply (regcache, i, &regs->fp_regs[i - fp0_regnum]);
    }
}

/* Collect register REGNUM in the floating point register set
   REGSET. from register cache REGCACHE into the buffer specified by
   FPREGS and LEN.  If REGNUM is -1, do this for all registers in
   REGSET.  */

static void
sh_linux_collect_fpregset (const struct regset *regset,
			   const struct regcache *regcache,
			   int regnum, void *fpregs, size_t len)
{
  struct gdbarch *gdbarch = get_regcache_arch (regcache);
  struct sh_linux_user_fpu_struct *regs = fpregs;
  int fp0_regnum = gdbarch_fp0_regnum (gdbarch);
  int i;

  gdb_assert (len >= sizeof (struct sh_linux_user_fpu_struct));

  if (regnum == FPSCR_REGNUM || regnum == -1)
    regcache_raw_collect (regcache, FPSCR_REGNUM, &regs->fpscr);

  if (regnum == FPUL_REGNUM || regnum == -1)
    regcache_raw_collect (regcache, FPUL_REGNUM, &regs->fpul);

  for (i = fp0_regnum; i <= (fp0_regnum + 15); i++)
    {
      if (regnum == i || regnum == -1)
	regcache_raw_collect (regcache, i, &regs->fp_regs[i - fp0_regnum]);
    }
}

/* SH register sets.  */

static struct regset sh_linux_gregset =
{
  NULL,
  sh_linux_supply_gregset,
  sh_linux_collect_gregset
};

static struct regset sh_linux_fpregset =
{
  NULL,
  sh_linux_supply_fpregset,
  sh_linux_collect_fpregset
};

/* Return the appropriate register set for the core section identified
   by SECT_NAME and SECT_SIZE.  */

static const struct regset *
sh_linux_regset_from_core_section (struct gdbarch *gdbarch,
				   const char *sect_name, size_t sect_size)
{
  if (strcmp (sect_name, ".reg") == 0
      && sect_size >= sizeof (struct sh_linux_pt_regs))
    return &sh_linux_gregset;
  else if (strcmp (sect_name, ".reg2") == 0
	   && sect_size >= sizeof (struct sh_linux_user_fpu_struct))
    return &sh_linux_fpregset;

  return NULL;
}

static void
sh_linux_init_abi (struct gdbarch_info info, struct gdbarch *gdbarch)
{
  /* GNU/Linux uses SVR4-style shared libraries.  */
  set_gdbarch_skip_trampoline_code (gdbarch, find_solib_trampoline_target);
  set_solib_svr4_fetch_link_map_offsets
    (gdbarch, svr4_ilp32_fetch_link_map_offsets);
  set_gdbarch_skip_solib_resolver (gdbarch, glibc_skip_solib_resolver);

  set_gdbarch_fetch_tls_load_module_address (gdbarch,
                                             svr4_fetch_objfile_link_map);

  /* Core files are supported for 32-bit SH only, at present`.  */
  if (info.bfd_arch_info->mach != bfd_mach_sh5)
    set_gdbarch_regset_from_core_section
      (gdbarch, sh_linux_regset_from_core_section);
}

/* Provide a prototype to silence -Wmissing-prototypes.  */
extern void _initialize_sh_linux_tdep (void);

void
_initialize_sh_linux_tdep (void)
{
  gdbarch_register_osabi (bfd_arch_sh, 0, GDB_OSABI_LINUX, sh_linux_init_abi);
}
